@@ -1,22 +1,26 @@
 #![recursion_limit = "128"]
+
+use std::{collections::HashMap, unreachable};
+
 use if_chain::if_chain;
 use proc_macro2::Span;
 use proc_macro_error::{abort, proc_macro_error};
-use quote::{quote, quote_spanned};
 use quote::ToTokens;
-use std::{collections::HashMap, unreachable};
+use quote::{quote, quote_spanned};
 use syn::{parse_quote, spanned::Spanned, GenericParam, Lifetime, LifetimeDef, Type};
-use validator_types::{CustomArgument, Validator};
-
-mod asserts;
-mod lit;
-mod quoting;
-mod validation;
 
 use asserts::{assert_has_len, assert_has_range, assert_string_type, assert_type_matches};
 use lit::*;
 use quoting::{quote_schema_validations, quote_validator, FieldQuoter};
 use validation::*;
+use validator_types::{CustomArgument, Validator};
+
+use crate::asserts::assert_custom_arg_type;
+
+mod asserts;
+mod lit;
+mod quoting;
+mod validation;
 
 #[proc_macro_derive(Validate, attributes(validate))]
 #[proc_macro_error]
@@ -27,11 +31,13 @@ pub fn derive_validation(input: proc_macro::TokenStream) -> proc_macro::TokenStr
 
 fn impl_validate(ast: &syn::DeriveInput) -> proc_macro2::TokenStream {
     // Collecting the validators
-    let mut validations = collect_field_validations(ast);
-    let (arg_type, has_arg) = construct_validator_argument_type(&mut validations);
-    let (validations, nested_validations) = quote_field_validations(validations);
+    let mut fields_validations = collect_field_validations(ast);
+    let mut struct_validations = find_struct_validations(&ast.attrs);
+    let (arg_type, has_arg) =
+        construct_validator_argument_type(&mut fields_validations, &mut struct_validations);
+    let (validations, nested_validations) = quote_field_validations(fields_validations);
 
-    let schema_validations = quote_schema_validations(&find_struct_validations(&ast.attrs));
+    let schema_validations = quote_schema_validations(&struct_validations);
 
     // Struct specific definitions
     let ident = &ast.ident;
@@ -56,6 +62,7 @@ fn impl_validate(ast: &syn::DeriveInput) -> proc_macro2::TokenStream {
     expanded_generic
         .params
         .insert(0, GenericParam::Lifetime(LifetimeDef::new(Lifetime::new("'v_a", ast.span()))));
+
     let (impl_generics, _, _) = expanded_generic.split_for_impl();
 
     // Implementing ValidateArgs
@@ -128,17 +135,23 @@ fn collect_field_validations(ast: &syn::DeriveInput) -> Vec<FieldInformation> {
 }
 
 fn construct_validator_argument_type(
-    validations: &mut Vec<FieldInformation>,
+    fields_validations: &mut Vec<FieldInformation>,
+    struct_validations: &mut Vec<SchemaValidation>,
 ) -> (proc_macro2::TokenStream, bool) {
     const ARGS_PARAMETER_NAME: &str = "args";
 
     // This iterator only holds custom validations with a argument_type
-    let mut customs: Vec<&mut CustomArgument> = validations
+    let mut customs: Vec<&mut CustomArgument> = fields_validations
         .iter_mut()
         .map(|x| x.validations.iter_mut().filter_map(|x| x.validator.get_custom_argument_mut()))
         .flatten()
         .collect();
 
+    let mut schemas: Vec<&mut CustomArgument> =
+        struct_validations.iter_mut().map(|x| x.args.as_mut()).flatten().collect();
+
+    customs.append(&mut schemas);
+
     if customs.is_empty() {
         // Just the default empty type if no types are defined
         (quote!(()), false)
@@ -146,7 +159,7 @@ fn construct_validator_argument_type(
         // A single parameter will not be wrapped in a tuple
         let arg = customs.pop().unwrap();
         arg.arg_access = Some(syn::parse_str(ARGS_PARAMETER_NAME).unwrap());
-        
+
         let type_stream: &Type = &arg.arg_type;
         let span = arg.def_span;
         (quote_spanned!(span=> #type_stream), true)
@@ -157,7 +170,7 @@ fn construct_validator_argument_type(
             let arg_access_string = format!("{}.{}", ARGS_PARAMETER_NAME, index);
             arg.arg_access = Some(syn::parse_str(arg_access_string.as_str()).unwrap());
             index += 1;
-            
+
             let type_stream: &Type = &arg.arg_type;
             let span = arg.def_span;
 
@@ -198,8 +211,7 @@ fn find_struct_validation(attr: &syn::Attribute) -> SchemaValidation {
 
     if_chain! {
         if let Ok(syn::Meta::List(syn::MetaList { ref nested, .. })) = attr.parse_meta();
-        if let syn::NestedMeta::Meta(ref item) = nested[0];
-        if let syn::Meta::List(syn::MetaList { ref path, ref nested, .. }) = *item;
+        if let syn::NestedMeta::Meta(syn::Meta::List(syn::MetaList { ref path, ref nested, .. })) = nested[0];
 
         then {
             let ident = path.get_ident().unwrap();
@@ -211,11 +223,11 @@ fn find_struct_validation(attr: &syn::Attribute) -> SchemaValidation {
             let mut skip_on_field_errors = true;
             let mut code = None;
             let mut message = None;
+            let mut args = None;
 
             for arg in nested {
                 if_chain! {
-                    if let syn::NestedMeta::Meta(ref item) = *arg;
-                    if let syn::Meta::NameValue(syn::MetaNameValue { ref path, ref lit, .. }) = *item;
+                    if let syn::NestedMeta::Meta(syn::Meta::NameValue(syn::MetaNameValue { ref path, ref lit, .. })) = *arg;
 
                     then {
                         let ident = path.get_ident().unwrap();
@@ -248,6 +260,24 @@ fn find_struct_validation(attr: &syn::Attribute) -> SchemaValidation {
                                     : only a string is allowed"),
                                 };
                             },
+                            "arg" => {
+                                match lit_to_string(lit) {
+                                    Some(s) => {
+                                        match syn::parse_str::<syn::Type>(s.as_str()) {
+                                            Ok(arg_type) => {
+                                                assert_custom_arg_type(&lit.span(), &arg_type);
+                                                args = Some(CustomArgument::new(lit.span(), arg_type));
+                                            }
+                                            Err(_) => {
+                                                let mut msg = "invalid argument type for `arg` of `schema` validator: The string has to be a single type.".to_string();
+                                                msg.push_str("\n(Tip: You can combine multiple types into one tuple.)");
+                                                error(lit.span(), msg.as_str());
+                                            }
+                                        }
+                                    }
+                                    None => error(lit.span(), "invalid argument type for `arg` of `custom` validator: expected a string")
+                                };
+                            },
                             _ => error(lit.span(), "Unknown argument")
                         }
                     } else {
@@ -262,6 +292,7 @@ fn find_struct_validation(attr: &syn::Attribute) -> SchemaValidation {
 
             SchemaValidation {
                 function,
+                args,
                 skip_on_field_errors,
                 code,
                 message,
@@ -277,7 +308,7 @@ fn find_struct_validations(struct_attrs: &[syn::Attribute]) -> Vec<SchemaValidat
     struct_attrs
         .iter()
         .filter(|attribute| attribute.path == parse_quote!(validate))
-        .map(|attribute| find_struct_validation(attribute))
+        .map(find_struct_validation)
         .collect()
 }
 
@@ -311,12 +342,7 @@ fn find_fields_type(fields: &[syn::Field]) -> HashMap<String, String> {
             _ => {
                 let mut field_type = proc_macro2::TokenStream::new();
                 field.ty.to_tokens(&mut field_type);
-                abort!(
-                    field.ty.span(),
-                    "Type `{}` of field `{}` not supported",
-                    field_type,
-                    field_ident
-                )
+                field_type.to_string().replace(' ', "")
             }
         };
 
@@ -431,7 +457,7 @@ fn find_validators_for_field(
                                         match lit_to_string(lit) {
                                             Some(s) => validators.push(FieldValidation::new(Validator::Custom {
                                                 function: s,
-                                                argument: None,
+                                                argument: Box::new(None),
                                             })),
                                             None => error(lit.span(), "invalid argument for `custom` validator: only strings are allowed"),
                                         };
@@ -451,9 +477,9 @@ fn find_validators_for_field(
                                     "must_match" => {
                                         match lit_to_string(lit) {
                                             Some(s) => {
-                                                assert_type_matches(rust_ident.clone(), field_type, field_types.get(&s), &attr);
+                                                assert_type_matches(rust_ident.clone(), field_type, field_types.get(&s), attr);
                                                 validators.push(FieldValidation::new(Validator::MustMatch(s)));
-                                            },
+                                            }
                                             None => error(lit.span(), "invalid argument for `must_match` validator: only strings are allowed"),
                                         };
                                     }
@@ -531,7 +557,7 @@ fn find_validators_for_field(
                                                 rust_ident.clone(),
                                                 field_type,
                                                 field_types.get(t2),
-                                                &attr,
+                                                attr,
                                             );
                                         }
                                         validators.push(validation);
@@ -554,12 +580,14 @@ fn find_validators_for_field(
                         field_ident
                     );
                 } else {
-                    abort!(attr.span(),
+                    abort!(
+                        attr.span(),
                         "Unable to parse this attribute for the field `{}` with the error: {:?}",
-                        field_ident, e
+                        field_ident,
+                        e
                     );
                 }
-            },
+            }
         }
 
         if has_validate && validators.is_empty() {
