@@ -1,6 +1,7 @@
 use if_chain::if_chain;
 use proc_macro2::{self, Span};
 use quote::quote;
+
 use validator_types::Validator;
 
 use crate::asserts::{COW_TYPE, NUMBER_TYPES};
@@ -31,7 +32,7 @@ impl FieldQuoter {
 
         if self._type.starts_with("Option<") {
             quote!(#ident)
-        } else if COW_TYPE.is_match(&self._type.as_ref()) {
+        } else if COW_TYPE.is_match(self._type.as_ref()) {
             quote!(self.#ident.as_ref())
         } else if self._type.starts_with('&') || NUMBER_TYPES.contains(&self._type.as_ref()) {
             quote!(self.#ident)
@@ -43,9 +44,12 @@ impl FieldQuoter {
     pub fn quote_validator_field(&self) -> proc_macro2::TokenStream {
         let ident = &self.ident;
 
-        if self._type.starts_with("Option<") || self._type.starts_with("Vec<") {
+        if self._type.starts_with("Option<")
+            || self._type.starts_with("Vec<")
+            || is_map(&self._type)
+        {
             quote!(#ident)
-        } else if COW_TYPE.is_match(&self._type.as_ref()) {
+        } else if COW_TYPE.is_match(self._type.as_ref()) {
             quote!(self.#ident.as_ref())
         } else {
             quote!(self.#ident)
@@ -88,7 +92,7 @@ impl FieldQuoter {
 
     /// Wrap the quoted output of a validation with a for loop if
     /// the field type is a vector
-    pub fn wrap_if_vector(&self, tokens: proc_macro2::TokenStream) -> proc_macro2::TokenStream {
+    pub fn wrap_if_collection(&self, tokens: proc_macro2::TokenStream) -> proc_macro2::TokenStream {
         let field_ident = &self.ident;
         let field_name = &self.name;
         if self._type.starts_with("Vec<") {
@@ -111,12 +115,45 @@ impl FieldQuoter {
                 }).collect();
                 result = ::validator::ValidationErrors::merge_all(result, #field_name, results);
             });
+        } else if is_map(&self._type) {
+            if self._type.starts_with("Option<") {
+                return quote!(
+                if !::validator::ValidationErrors::has_error(&result, #field_name) {
+                    let results: Vec<_> = #field_ident.iter().map(|(_, #field_ident)| {
+                        let mut result = ::std::result::Result::Ok(());
+                        #tokens
+                        result
+                    }).collect();
+                    result = ::validator::ValidationErrors::merge_all(result, #field_name, results);
+                });
+            } else {
+                return quote!(
+                if !::validator::ValidationErrors::has_error(&result, #field_name) {
+                    let results: Vec<_> = self.#field_ident.iter().map(|(_, #field_ident)| {
+                        let mut result = ::std::result::Result::Ok(());
+                        #tokens
+                        result
+                    }).collect();
+                    result = ::validator::ValidationErrors::merge_all(result, #field_name, results);
+                });
+            }
         }
 
         tokens
     }
 }
 
+fn is_map(_type: &str) -> bool {
+    if _type.starts_with("Option<") {
+        return is_map(&_type[7..]);
+    }
+
+    _type.starts_with("HashMap<")
+        || _type.starts_with("FxHashMap<")
+        || _type.starts_with("FnvHashMap<")
+        || _type.starts_with("BTreeMap<")
+}
+
 /// Quote an actual end-user error creation automatically
 fn quote_error(validation: &FieldValidation) -> proc_macro2::TokenStream {
     let code = &validation.code;
@@ -160,16 +197,16 @@ pub fn quote_length_validation(
         };
 
         let min_tokens = option_to_tokens(
-            &min.clone().map(|ref x| value_or_path_to_tokens(x)).map(|x| quote!(#x as u64)),
+            &min.clone().as_ref().map(value_or_path_to_tokens).map(|x| quote!(#x as u64)),
         );
         let max_tokens = option_to_tokens(
-            &max.clone().map(|ref x| value_or_path_to_tokens(x)).map(|x| quote!(#x as u64)),
+            &max.clone().as_ref().map(value_or_path_to_tokens).map(|x| quote!(#x as u64)),
         );
         let equal_tokens = option_to_tokens(
-            &equal.clone().map(|ref x| value_or_path_to_tokens(x)).map(|x| quote!(#x as u64)),
+            &equal.clone().as_ref().map(value_or_path_to_tokens).map(|x| quote!(#x as u64)),
         );
 
-        let quoted_error = quote_error(&validation);
+        let quoted_error = quote_error(validation);
         let quoted = quote!(
             if !::validator::validate_length(
                 #validator_param,
@@ -222,7 +259,7 @@ pub fn quote_range_validation(
             max.clone().map(|x| value_or_path_to_tokens(&x)).map(|x| quote!(#x as f64));
         let max_tokens = option_to_tokens(&max_tokens);
 
-        let quoted_error = quote_error(&validation);
+        let quoted_error = quote_error(validation);
         let quoted = quote!(
             if !::validator::validate_range(
                 #quoted_ident as f64,
@@ -251,7 +288,7 @@ pub fn quote_credit_card_validation(
     let field_name = &field_quoter.name;
     let validator_param = field_quoter.quote_validator_param();
 
-    let quoted_error = quote_error(&validation);
+    let quoted_error = quote_error(validation);
     let quoted = quote!(
         if !::validator::validate_credit_card(#validator_param) {
             #quoted_error
@@ -271,7 +308,7 @@ pub fn quote_phone_validation(
     let field_name = &field_quoter.name;
     let validator_param = field_quoter.quote_validator_param();
 
-    let quoted_error = quote_error(&validation);
+    let quoted_error = quote_error(validation);
     let quoted = quote!(
         if !::validator::validate_phone(#validator_param) {
             #quoted_error
@@ -291,7 +328,7 @@ pub fn quote_non_control_character_validation(
     let field_name = &field_quoter.name;
     let validator_param = field_quoter.quote_validator_param();
 
-    let quoted_error = quote_error(&validation);
+    let quoted_error = quote_error(validation);
     let quoted = quote!(
         if !::validator::validate_non_control_character(#validator_param) {
             #quoted_error
@@ -310,7 +347,7 @@ pub fn quote_url_validation(
     let field_name = &field_quoter.name;
     let validator_param = field_quoter.quote_validator_param();
 
-    let quoted_error = quote_error(&validation);
+    let quoted_error = quote_error(validation);
     let quoted = quote!(
         if !::validator::validate_url(#validator_param) {
             #quoted_error
@@ -329,7 +366,7 @@ pub fn quote_email_validation(
     let field_name = &field_quoter.name;
     let validator_param = field_quoter.quote_validator_param();
 
-    let quoted_error = quote_error(&validation);
+    let quoted_error = quote_error(validation);
     let quoted = quote!(
         if !::validator::validate_email(#validator_param) {
             #quoted_error
@@ -350,7 +387,7 @@ pub fn quote_must_match_validation(
 
     if let Validator::MustMatch(ref other) = validation.validator {
         let other_ident = syn::Ident::new(other, Span::call_site());
-        let quoted_error = quote_error(&validation);
+        let quoted_error = quote_error(validation);
         let quoted = quote!(
             if !::validator::validate_must_match(&self.#ident, &self.#other_ident) {
                 #quoted_error
@@ -377,7 +414,7 @@ pub fn quote_custom_validation(
         let fn_ident: syn::Path = syn::parse_str(function).unwrap();
 
         let access = if_chain! {
-            if let Some(argument) = argument;
+            if let Some(argument) = &**argument;
             if let Some(access) = &argument.arg_access;
             then {
                 quote!(, #access)
@@ -417,7 +454,7 @@ pub fn quote_contains_validation(
     let validator_param = field_quoter.quote_validator_param();
 
     if let Validator::Contains(ref needle) = validation.validator {
-        let quoted_error = quote_error(&validation);
+        let quoted_error = quote_error(validation);
         let quoted = quote!(
             if !::validator::validate_contains(#validator_param, &#needle) {
                 #quoted_error
@@ -442,7 +479,7 @@ pub fn quote_regex_validation(
 
     if let Validator::Regex(ref re) = validation.validator {
         let re_ident: syn::Path = syn::parse_str(re).unwrap();
-        let quoted_error = quote_error(&validation);
+        let quoted_error = quote_error(validation);
         let quoted = quote!(
             if !#re_ident.is_match(#validator_param) {
                 #quoted_error
@@ -461,7 +498,7 @@ pub fn quote_nested_validation(field_quoter: &FieldQuoter) -> proc_macro2::Token
     let field_name = &field_quoter.name;
     let validator_field = field_quoter.quote_validator_field();
     let quoted = quote!(result = ::validator::ValidationErrors::merge(result, #field_name, #validator_field.validate()););
-    field_quoter.wrap_if_option(field_quoter.wrap_if_vector(quoted))
+    field_quoter.wrap_if_option(field_quoter.wrap_if_collection(quoted))
 }
 
 pub fn quote_validator(
@@ -472,36 +509,36 @@ pub fn quote_validator(
 ) {
     match validation.validator {
         Validator::Length { .. } => {
-            validations.push(quote_length_validation(&field_quoter, validation))
+            validations.push(quote_length_validation(field_quoter, validation))
         }
         Validator::Range { .. } => {
-            validations.push(quote_range_validation(&field_quoter, validation))
+            validations.push(quote_range_validation(field_quoter, validation))
         }
-        Validator::Email => validations.push(quote_email_validation(&field_quoter, validation)),
-        Validator::Url => validations.push(quote_url_validation(&field_quoter, validation)),
+        Validator::Email => validations.push(quote_email_validation(field_quoter, validation)),
+        Validator::Url => validations.push(quote_url_validation(field_quoter, validation)),
         Validator::MustMatch(_) => {
-            validations.push(quote_must_match_validation(&field_quoter, validation))
+            validations.push(quote_must_match_validation(field_quoter, validation))
         }
         Validator::Custom { .. } => {
-            validations.push(quote_custom_validation(&field_quoter, validation))
+            validations.push(quote_custom_validation(field_quoter, validation))
         }
         Validator::Contains(_) => {
-            validations.push(quote_contains_validation(&field_quoter, validation))
+            validations.push(quote_contains_validation(field_quoter, validation))
         }
-        Validator::Regex(_) => validations.push(quote_regex_validation(&field_quoter, validation)),
+        Validator::Regex(_) => validations.push(quote_regex_validation(field_quoter, validation)),
         #[cfg(feature = "card")]
         Validator::CreditCard => {
-            validations.push(quote_credit_card_validation(&field_quoter, validation))
+            validations.push(quote_credit_card_validation(field_quoter, validation))
         }
         #[cfg(feature = "phone")]
-        Validator::Phone => validations.push(quote_phone_validation(&field_quoter, validation)),
-        Validator::Nested => nested_validations.push(quote_nested_validation(&field_quoter)),
+        Validator::Phone => validations.push(quote_phone_validation(field_quoter, validation)),
+        Validator::Nested => nested_validations.push(quote_nested_validation(field_quoter)),
         #[cfg(feature = "unic")]
         Validator::NonControlCharacter => {
-            validations.push(quote_non_control_character_validation(&field_quoter, validation))
+            validations.push(quote_non_control_character_validation(field_quoter, validation))
         }
         Validator::Required | Validator::RequiredNested => {
-            validations.push(quote_required_validation(&field_quoter, validation))
+            validations.push(quote_required_validation(field_quoter, validation))
         }
     }
 }
@@ -509,6 +546,13 @@ pub fn quote_validator(
 pub fn quote_schema_validation(v: &SchemaValidation) -> proc_macro2::TokenStream {
     let fn_ident: syn::Path = syn::parse_str(&v.function).unwrap();
 
+    let arg_quoted = if let Some(ref args) = v.args {
+        let arg_type = &args.arg_access;
+        quote!(self, #arg_type)
+    } else {
+        quote!(self)
+    };
+
     let add_message_quoted = if let Some(ref m) = v.message {
         quote!(err.message = Some(::std::borrow::Cow::from(#m));)
     } else {
@@ -518,7 +562,7 @@ pub fn quote_schema_validation(v: &SchemaValidation) -> proc_macro2::TokenStream
     let mut_err_token = if v.message.is_some() { quote!(mut) } else { quote!() };
 
     let quoted = quote!(
-        match #fn_ident(self) {
+        match #fn_ident(#arg_quoted) {
             ::std::result::Result::Ok(()) => (),
             ::std::result::Result::Err(#mut_err_token err) => {
                 #add_message_quoted
@@ -548,7 +592,7 @@ pub fn quote_required_validation(
     let ident = &field_quoter.ident;
     let validator_param = quote!(&self.#ident);
 
-    let quoted_error = quote_error(&validation);
+    let quoted_error = quote_error(validation);
     let quoted = quote!(
         if !::validator::validate_required(#validator_param) {
             #quoted_error
